import json
import mock
import re
import ssl
import unittest
from KubeLibrary import KubeLibrary
from KubeLibrary.exceptions import BearerTokenWithPrefixException
from kubernetes.config.config_exception import ConfigException
from urllib3_mock import Responses


class AttributeDict(object):
    """
    Based on http://databio.org/posts/python_AttributeDict.html

    A class to convert a nested Dictionary into an object with key-values
    accessibly using attribute notation (AttributeDict.attribute) instead of
    key notation (Dict["key"]). This class recursively sets Dicts to objects,
    allowing you to recurse down nested dicts (like: AttributeDict.attr.attr)
    """
    def __init__(self, entries):
        self.add_entries(entries)

    def add_entries(self, entries):
        self._root = entries
        for key, value in entries.items():
            if type(value) is dict:
                self.__dict__[key] = AttributeDict(value)
            elif type(value) is list:
                self.__dict__[key] = [AttributeDict(item) if type(item) is dict else item for item in value]
            else:
                self.__dict__[key] = value

    def __iter__(self):
        return iter(self._root)

    def __getitem__(self, key):
        """
        Provides dict-style access to attributes
        """
        return getattr(self, key)


def mock_read_daemonset_details_in_namespace(name, namespace):
    if namespace == 'default':
        with open('test/resources/daemonset_details.json') as json_file:
            daemonset_details_content = json.load(json_file)
            read_daemonset_details = AttributeDict({'items': daemonset_details_content})
            return read_daemonset_details


def mock_read_service_details_in_namespace(name, namespace):
    if namespace == 'default':
        with open('test/resources/service_details.json') as json_file:
            service_details_content = json.load(json_file)
            read_service_details = AttributeDict({'items': service_details_content})
            return read_service_details


def mock_read_hpa_details_in_namespace(name, namespace):
    if namespace == 'default':
        with open('test/resources/hpa_details.json') as json_file:
            hpa_details_content = json.load(json_file)
            read_hpa_details = AttributeDict({'items': hpa_details_content})
            return read_hpa_details


def mock_read_ingress_details_in_namespace(name, namespace):
    if namespace == 'default':
        with open('test/resources/ingress_details.json') as json_file:
            ingress_details_content = json.load(json_file)
            read_ingress_details = AttributeDict({'items': ingress_details_content})
            return read_ingress_details


def mock_read_cron_job_details_in_namespace(name, namespace):
    if namespace == 'default':
        with open('test/resources/cronjob_details.json') as json_file:
            cron_job_details_content = json.load(json_file)
            read_cron_job_details = AttributeDict({'items': cron_job_details_content})
            return read_cron_job_details


def mock_list_namespaced_daemonsets(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/daemonset.json') as json_file:
            daemonsets_content = json.load(json_file)
            list_of_daemonsets = AttributeDict({'items': daemonsets_content})
            return list_of_daemonsets


def mock_list_namespaced_cronjobs(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/cronjob.json') as json_file:
            cronjobs_content = json.load(json_file)
            list_of_cronjobs = AttributeDict({'items': cronjobs_content})
            return list_of_cronjobs


def mock_list_namespaced_ingresses(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/ingress.json') as json_file:
            ingresses_content = json.load(json_file)
            list_ingresses = AttributeDict({'items': ingresses_content})
            return list_ingresses


def mock_read_namespaced_endpoints(name, namespace):
    if namespace == 'default':
        with open('test/resources/endpoints.json') as json_file:
            endpoints_content = json.load(json_file)
            read_endpoints = AttributeDict({'items': endpoints_content})
            return read_endpoints


def mock_list_namespaced_config_map(namespace, watch=False, label_selector=""):
    with open('test/resources/configmap.json') as json_file:
        configmap_content = json.load(json_file)
        configmap = AttributeDict({'items': configmap_content})
        return configmap


def mock_list_namespaced_deployments(namespace, watch=False, label_selector=""):
    with open('test/resources/deployment.json') as json_file:
        deployments_content = json.load(json_file)
        deployments = AttributeDict({'items': deployments_content})
        return deployments


def mock_list_namespaced_replicasets(namespace, watch=False, label_selector=""):
    with open('test/resources/replicaset.json') as json_file:
        replicasets_content = json.load(json_file)
        replicasets = AttributeDict({'items': replicasets_content})
        return replicasets


def mock_list_pvc(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/pvc.json') as json_file:
            pvc_content = json.load(json_file)
            list_pvc = AttributeDict({'items': pvc_content})
            return list_pvc


def mock_list_cluster_roles(watch=False):
    with open('test/resources/cluster_role.json') as json_file:
        cluster_roles_content = json.load(json_file)
        list_of_cluster_roles = AttributeDict({'items': cluster_roles_content})
        return list_of_cluster_roles


def mock_list_namespaced_services(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/service.json') as json_file:
            services_content = json.load(json_file)
            list_services = AttributeDict({'items': services_content})
            return list_services


def mock_list_namespaced_hpas(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/hpa.json') as json_file:
            hpas_content = json.load(json_file)
            list_hpas = AttributeDict({'items': hpas_content})
            return list_hpas


def mock_list_namespaced_pod(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/pods.json') as json_file:
            pods_content = json.load(json_file)
            list_of_pods = AttributeDict({'items': pods_content})
            return list_of_pods


def mock_read_namespaced_pod_status(name, namespace):
    if namespace == 'default':
        with open('test/resources/pod_status.json') as json_file:
            pod_content = json.load(json_file)
            pod_status = AttributeDict({'status': pod_content['status']})
            return pod_status


def mock_list_cluster_role_bindings(watch=False):
    with open('test/resources/cluster_role_bind.json') as json_file:
        cluster_role_bindings_content = json.load(json_file)
        list_of_cluster_role_bindings = AttributeDict({'items': cluster_role_bindings_content})
        return list_of_cluster_role_bindings


def mock_list_namespaced_service_accounts(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/service_accounts.json') as json_file:
            service_accounts_content = json.load(json_file)
            list_of_service_accounts = AttributeDict({'items': service_accounts_content})
            return list_of_service_accounts


def mock_list_namespaced_jobs(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/jobs.json') as json_file:
            jobs_content = json.load(json_file)
            list_of_jobs = AttributeDict({'items': jobs_content})
            return list_of_jobs


def mock_list_namespaced_secrets(namespace, watch=False, label_selector=""):
    if namespace == 'default':
        with open('test/resources/secrets.json') as json_file:
            secrets_content = json.load(json_file)
            list_of_secrets = AttributeDict({'items': secrets_content})
            return list_of_secrets


def mock_list_namespaces(watch=False, label_selector=""):
    with open('test/resources/namespaces.json') as json_file:
        namespaces_content = json.load(json_file)
        list_of_namespaces = AttributeDict({'items': namespaces_content})
        return list_of_namespaces


def mock_list_node_info(watch=False, label_selector=""):
    with open('test/resources/node_info.json') as json_file:
        node_info_content = json.load(json_file)
        node_info = AttributeDict(node_info_content)
        return node_info


def mock_list_namespaced_roles(namespace, watch=False):
    if namespace == 'default':
        with open('test/resources/role.json') as json_file:
            role_content = json.load(json_file)
            list_of_role = AttributeDict({'items': role_content})
            return list_of_role


def mock_list_namespaced_role_bindings(namespace, watch=False):
    if namespace == 'default':
        with open('test/resources/rolebinding.json') as json_file:
            role_bind_content = json.load(json_file)
            list_of_role_bind = AttributeDict({'items': role_bind_content})
            return list_of_role_bind


bearer_token = 'eyJhbGciOiJSUzI1NiIsImtpZCI6IjdXVWJMOUdTaDB1TjcyNmF0Sjk4RWlzQ05RaWdSUFoyN004TmlGT1pSX28ifQ.' \
               'eyJpc3MiOiJrdWJlcm5ldGVzL3NlcnZpY2VhY2NvdW50Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9uYW1' \
               'lc3BhY2UiOiJkZWZhdWx0Iiwia3ViZXJuZXRlcy5pby9zZXJ2aWNlYWNjb3VudC9zZWNyZXQubmFtZSI6Im15c2EtdG' \
               '9rZW4taDRzNzUiLCJrdWJlcm5ldGVzLmlvL3NlcnZpY2VhY2NvdW50L3NlcnZpY2UtYWNjb3VudC5uYW1lIjoibXlzY' \
               'SIsImt1YmVybmV0ZXMuaW8vc2VydmljZWFjY291bnQvc2VydmljZS1hY2NvdW50LnVpZCI6IjY5MTk5ZmUyLTIzNWIt' \
               'NGY3MC04MjEwLTkzZTk2YmM5ZmEwOCIsInN1YiI6InN5c3RlbTpzZXJ2aWNlYWNjb3VudDpkZWZhdWx0Om15c2EifQ.' \
               'V8VIYZ0B2y2h9p-2LTZ19klSuZ37HUWi-8F1yjfFTq83R1Dmax6DoDr5gWbVL4A054q5k1L2U12d50gox0V_kVsRTb3' \
               'KQnRSGCz1YgCqOVNLqnnsyu3kcmDaUDrFlJ4PuZ7R4DfvGCdK-BU9pj2MhcQT-tyfbGR-dwwkjwXTCPRZVW-CUm4qwY' \
               'bCGTpGbNXPXbEKtseXIxMkRg70Kav3M-YB1LYHQRx_T2IqKAmyhXlbMc8boqoEiSi6TRbMjZ9Yz-nkc82e6kAdc1O2F' \
               '4kFw-14kg2mX7Hu-02vob_LZmfR08UGu6VTkcfVK5VqZVg2oVBI4swZghQl8_fOtlplOg'

ca_cert = '/path/to/certificate.crt'

k8s_api_url = 'https://0.0.0.0:38041'

responses = Responses('requests.packages.urllib3')


class TestKubeLibrary(unittest.TestCase):

    apis = ('v1', 'extensionsv1beta1', 'batchv1', 'appsv1', 'batchv1_beta1',
            'custom_object', 'rbac_authv1_api', 'autoscalingv1', 'dynamic')

    @responses.activate
    def test_KubeLibrary_inits_from_kubeconfig(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        KubeLibrary(kube_config='test/resources/k3d')

    @responses.activate
    def test_KubeLibrary_inits_with_context(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        KubeLibrary(kube_config='test/resources/multiple_context', context='k3d-k3d-cluster2')

    @responses.activate
    def test_KubeLibrary_fails_for_wrong_context(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(kube_config='test/resources/multiple_context')
        self.assertRaises(ConfigException, kl.reload_config, kube_config='test/resources/multiple_context', context='k3d-k3d-cluster2-wrong')

    @responses.activate
    def test_inits_all_api_clients(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(kube_config='test/resources/k3d')
        for api in TestKubeLibrary.apis:
            self.assertIsNotNone(getattr(kl, api))

    @responses.activate
    def test_KubeLibrary_inits_without_cert_validation(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(kube_config='test/resources/k3d', cert_validation=False)
        for api in TestKubeLibrary.apis:
            target = getattr(kl, api)
            self.assertEqual(target.api_client.rest_client.pool_manager.connection_pool_kw['cert_reqs'], ssl.CERT_NONE)

    @responses.activate
    def test_KubeLibrary_inits_with_bearer_token(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(api_url=k8s_api_url, bearer_token=bearer_token)
        for api in TestKubeLibrary.apis:
            target = getattr(kl, api)
            self.assertEqual(kl.api_client.configuration.api_key, target.api_client.configuration.api_key)
        self.assertEqual(kl.api_client.configuration.ssl_ca_cert, None)

    @responses.activate
    def test_inits_with_bearer_token_raises_BearerTokenWithPrefixException(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(api_url=k8s_api_url, bearer_token=bearer_token)
        self.assertRaises(BearerTokenWithPrefixException, kl.reload_config, api_url=k8s_api_url, bearer_token='Bearer prefix should fail')

    @responses.activate
    def test_KubeLibrary_inits_with_bearer_token_with_ca_crt(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        kl = KubeLibrary(api_url=k8s_api_url, bearer_token=bearer_token, ca_cert=ca_cert)
        self.assertEqual(kl.api_client.configuration.ssl_ca_cert, ca_cert)
        self.assertEqual(kl.dynamic.configuration.ssl_ca_cert, ca_cert)
        self.assertEqual(kl.dynamic.client.configuration.ssl_ca_cert, ca_cert)

    @responses.activate
    def test_KubeLibrary_dynamic_init(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        kl = KubeLibrary(kube_config='test/resources/k3d')
        resource = kl.get_dynamic_resource("v1", "Pod")
        self.assertTrue(hasattr(resource, "get"))
        self.assertTrue(hasattr(resource, "watch"))
        self.assertTrue(hasattr(resource, "delete"))
        self.assertTrue(hasattr(resource, "create"))
        self.assertTrue(hasattr(resource, "patch"))
        self.assertTrue(hasattr(resource, "replace"))

    @responses.activate
    def test_KubeLibrary_dynamic_get(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        responses.add("GET", "/api/v1/mock/Mock", status=200,
                      body='{"api_version": "v1", "kind": "Pod", "name": "Mock", "msg": "My Mock Pod"}',
                      content_type="application/json")
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pod = kl.get("v1", "Pod", name="Mock")
        self.assertEqual(pod.msg, "My Mock Pod")

    @responses.activate
    def test_KubeLibrary_dynamic_patch(self):
        def mock_callback(request):
            self.assertEqual(request.body, '{"msg": "Mock"}')
            return (200, None, None)
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        responses.add_callback("PATCH", "/api/v1/mock/Mock", callback=mock_callback)
        kl = KubeLibrary(kube_config='test/resources/k3d')
        kl.patch("v1", "Pod", name="Mock", body={"msg": "Mock"})

    @responses.activate
    def test_KubeLibrary_dynamic_replace(self):
        def mock_callback(request):
            self.assertEqual(request.body, '{"msg": "Mock"}')
            return (200, None, None)
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        responses.add_callback("PUT", "/api/v1/mock/Mock", callback=mock_callback)
        kl = KubeLibrary(kube_config='test/resources/k3d')
        kl.replace("v1", "Pod", name="Mock", body={"msg": "Mock"})

    @responses.activate
    def test_KubeLibrary_dynamic_create(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        responses.add("POST", "/api/v1/mock", status=200)
        kl = KubeLibrary(kube_config='test/resources/k3d')
        kl.create("v1", "Pod", name="Mock")

    @responses.activate
    def test_KubeLibrary_dynamic_delete(self):
        responses.add("GET", "/version", status=200)
        responses.add("GET", "/apis", status=200, body='{"groups": [], "kind": "Pod" }', content_type="application/json")
        responses.add("GET", "/api/v1", status=200,
                      body='{"resources": [{"api_version": "v1", "kind": "Pod", "name": "Mock"}], "kind": "Pod"}',
                      content_type="application/json")
        responses.add("DELETE", "/api/v1/mock/Mock", status=200)
        kl = KubeLibrary(kube_config='test/resources/k3d')
        kl.delete("v1", "Pod", name="Mock")

    def test_generate_alphanumeric_str(self):
        name = KubeLibrary.generate_alphanumeric_str(10)
        self.assertEqual(10, len(name))

    def test_evaluate_callable_from_k8s_client(self):
        configmap = KubeLibrary.evaluate_callable_from_k8s_client(
            attr_name="V1ConfigMap",
            data={"msg": "Mock"}, api_version="v1", kind="ConfigMap",
            metadata=KubeLibrary.evaluate_callable_from_k8s_client(attr_name="V1ObjectMeta", name="Mock")
        )
        self.assertIsNotNone(configmap)
        self.assertEqual(configmap.metadata.name, "Mock")

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_list_namespaced_pod_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('.*', 'default')
        pods2 = kl.get_pods_in_namespace('.*', 'default')
        pods3 = kl.get_pod_names_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(pods), pods3)
        self.assertEqual(kl.filter_names(pods), kl.filter_pods_names(pods2))
        self.assertEqual(['octopus-0', 'grafana-5d9895c6c4-sfsn8'], kl.filter_names(pods))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_get_matching_pods_in_namespace(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('graf.*', 'default')
        self.assertEqual(['grafana-5d9895c6c4-sfsn8'], kl.filter_names(pods))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_filter_pods_containers_by_name(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('octopus.*', 'default')
        self.assertEqual('manager', kl.filter_pods_containers_by_name(pods, '.*')[0].name)

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_filter_containers_images(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('octopus.*', 'default')
        containers = kl.filter_pods_containers_by_name(pods, '.*')
        self.assertEqual(['eu.gcr.io/kyma-project/incubator/develop/octopus:dc5dc284'], kl.filter_containers_images(containers))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_filter_pods_containers_statuses_by_name(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('octopus.*', 'default')
        self.assertEqual(0, kl.filter_pods_containers_statuses_by_name(pods, '.*')[0].restart_count)

    @mock.patch('kubernetes.client.CoreV1Api.read_namespaced_pod_status')
    def test_read_namespaced_pod_status(self, mock_lnp):
        mock_lnp.side_effect = mock_read_namespaced_pod_status
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pod_status = kl.read_namespaced_pod_status('grafana-6769d4b669-fhspj', 'default')
        self.assertEqual('Running', pod_status['phase'])

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_pod')
    def test_filter_containers_resources(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_pod
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pods = kl.list_namespaced_pod_by_pattern('octopus.*', 'default')
        containers = kl.filter_pods_containers_by_name(pods, '.*')
        self.assertEqual('100m', kl.filter_containers_resources(containers)[0].limits.cpu)

    def test_assert_pod_has_labels(self):
        pod = mock_list_namespaced_pod('default').items[0]
        kl = KubeLibrary(kube_config='test/resources/k3d')
        labels = '{}'
        self.assertTrue(kl.assert_pod_has_labels(pod, labels))
        labels = '{"app":"octopus"}'
        self.assertTrue(kl.assert_pod_has_labels(pod, labels))
        labels = '{"app":"wrong"}'
        self.assertFalse(kl.assert_pod_has_labels(pod, labels))
        labels = '{"notexists":"octopus"}'
        self.assertFalse(kl.assert_pod_has_labels(pod, labels))
        labels = '{"badlyformatted:}'
        self.assertTrue(kl.assert_pod_has_labels(pod, labels) is False)

    def test_assert_pod_has_annotations(self):
        pod = mock_list_namespaced_pod('default').items[1]
        kl = KubeLibrary(kube_config='test/resources/k3d')
        labels = '{}'
        self.assertTrue(kl.assert_pod_has_annotations(pod, labels))
        labels = '{"checksum/config":"1c42968a1b9eca0bafc3273ca39c4705fe71dc632e721db9e8ce44ab1b8e1428"}'
        self.assertTrue(kl.assert_pod_has_annotations(pod, labels))
        labels = '{"checksum/config":"wrong"}'
        self.assertFalse(kl.assert_pod_has_annotations(pod, labels))
        labels = '{"notexists":"1c42968a1b9eca0bafc3273ca39c4705fe71dc632e721db9e8ce44ab1b8e1428"}'
        self.assertFalse(kl.assert_pod_has_annotations(pod, labels))
        labels = '{"badlyformatted:}'
        self.assertTrue(kl.assert_pod_has_annotations(pod, labels) is False)

    def test_assert_container_has_env_vars(self):
        pod = mock_list_namespaced_pod('default').items[0]
        kl = KubeLibrary(kube_config='test/resources/k3d')
        container = kl.filter_pods_containers_by_name([pod], '.*')[0]
        env_vars = '{}'
        self.assertTrue(kl.assert_container_has_env_vars(container, env_vars))
        env_vars = '{"SECRET_NAME":"webhook-server-secret"}'
        self.assertTrue(kl.assert_container_has_env_vars(container, env_vars))
        env_vars = '{"SECRET_NAME":"wrong"}'
        self.assertFalse(kl.assert_container_has_env_vars(container, env_vars))
        env_vars = '{"NOT_EXISTING":"wrong"}'
        self.assertFalse(kl.assert_container_has_env_vars(container, env_vars))
        env_vars = '{"badlyformatted:}'
        self.assertFalse(kl.assert_container_has_env_vars(container, env_vars))

    @unittest.skip("Will overwrite *.json")
    def test_gather_pods_obejcts_to_json(self):
        kl = KubeLibrary(kube_config='~/.kube/k3d')
        ret = kl.v1.read_namespaced_pod_status('grafana-6769d4b669-fhspj', 'default')
        pods_str = str(ret).replace("'", '"') \
                           .replace('None', 'null') \
                           .replace('True', 'true') \
                           .replace('False', 'false')
        # serialize datetime into fixed timestamp
        pods = re.sub(r'datetime(.+?)\)\)', '1592598289', pods_str)
        print(pods)
        with open('test/resources/pod_status.json', 'w') as outfile:
            json.dump(json.loads(pods), outfile, indent=4)

    @mock.patch('kubernetes.client.CoreV1Api.list_namespace')
    def test_list_namespace(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaces
        kl = KubeLibrary(kube_config='test/resources/k3d')
        namespaces = kl.list_namespace()
        namespaces2 = kl.get_namespaces()
        self.assertEqual(kl.filter_names(namespaces), namespaces2)
        self.assertTrue(len(namespaces) > 0)
        self.assertEqual(['default', 'kubelib-test-test-objects-chart'], kl.filter_names(namespaces))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_service_account')
    def test_list_namespaced_service_account_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_service_accounts
        kl = KubeLibrary(kube_config='test/resources/k3d')
        sa = kl.list_namespaced_service_account_by_pattern('.*', 'default')
        sa2 = kl.get_service_accounts_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(sa), kl.filter_service_accounts_names(sa2))
        self.assertEqual(['default', 'kubelib-test-test-objects-chart'], kl.filter_names(sa))

    @mock.patch('kubernetes.client.CoreV1Api.list_node')
    def test_get_kubelet_version(self, mock_lnp):
        mock_lnp.side_effect = mock_list_node_info
        kl = KubeLibrary(kube_config='test/resources/k3d')
        kl_version = kl.get_kubelet_version()
        self.assertTrue(len(kl_version) > 0)
        self.assertEqual(['v1.20.0+k3s2'], kl_version)

    @mock.patch('kubernetes.client.BatchV1Api.list_namespaced_job')
    def test_list_namespaced_job_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_jobs
        kl = KubeLibrary(kube_config='test/resources/k3d')
        jobs = kl.list_namespaced_job_by_pattern('.*', 'default')
        jobs2 = kl.get_jobs_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(jobs), kl.filter_names(jobs2))
        self.assertEqual(['octopus-0', 'octopus-1', 'octopus-2', 'octopus-3'], kl.filter_names(jobs))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_secret')
    def test_list_namespaced_secret_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_secrets
        kl = KubeLibrary(kube_config='test/resources/k3d')
        secrets = kl.list_namespaced_secret_by_pattern('.*', 'default')
        secrets2 = kl.get_secrets_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(secrets), kl.filter_names(secrets2))
        self.assertEqual(['grafana'], kl.filter_names(secrets))

    @mock.patch('kubernetes.client.RbacAuthorizationV1Api.list_cluster_role')
    def test_list_cluster_role(self, mock_lnp):
        mock_lnp.side_effect = mock_list_cluster_roles
        kl = KubeLibrary(kube_config='test/resources/k3d')
        cluster_roles = kl.list_cluster_role()
        cluster_roles2 = kl.get_cluster_roles()
        self.assertEqual(kl.filter_names(cluster_roles), cluster_roles2)
        self.assertEqual(['secret-reader'], kl.filter_names(cluster_roles))

    @mock.patch('kubernetes.client.RbacAuthorizationV1Api.list_cluster_role_binding')
    def test_list_cluster_role_binding(self, mock_lnp):
        mock_lnp.side_effect = mock_list_cluster_role_bindings
        kl = KubeLibrary(kube_config='test/resources/k3d')
        cluster_role_bindings = kl.list_cluster_role_binding()
        cluster_role_bindings2 = kl.get_cluster_role_bindings()
        self.assertEqual(kl.filter_names(cluster_role_bindings), cluster_role_bindings2)
        self.assertEqual(['read-secrets-global'], kl.filter_names(cluster_role_bindings))

    @mock.patch('kubernetes.client.RbacAuthorizationV1Api.list_namespaced_role')
    def test_list_namespaced_role(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_roles
        kl = KubeLibrary(kube_config='test/resources/k3d')
        roles = kl.list_namespaced_role('default')
        roles2 = kl.get_roles_in_namespace('default')
        self.assertEqual(kl.filter_names(roles), roles2)
        self.assertEqual(['pod-reader'], kl.filter_names(roles))

    @mock.patch('kubernetes.client.RbacAuthorizationV1Api.list_namespaced_role_binding')
    def test_list_namespaced_role_binding(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_role_bindings
        kl = KubeLibrary(kube_config='test/resources/k3d')
        role_bindings = kl.list_namespaced_role_binding('default')
        role_bindings2 = kl.get_role_bindings_in_namespace('default')
        self.assertEqual(kl.filter_names(role_bindings), role_bindings2)
        self.assertEqual(['read-pods'], kl.filter_names(role_bindings))

    @mock.patch('kubernetes.client.AppsV1Api.list_namespaced_deployment')
    def test_list_namespaced_deployment_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_deployments
        kl = KubeLibrary(kube_config='test/resources/k3d')
        deployments = kl.list_namespaced_deployment_by_pattern('.*', 'default')
        deployments2 = kl.get_deployments_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(deployments), kl.filter_deployments_names(deployments2))
        self.assertEqual(['nginx-deployment'], kl.filter_names(deployments))

    @mock.patch('kubernetes.client.AppsV1Api.list_namespaced_replica_set')
    def test_list_namespaced_replica_set_by_pattern(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_replicasets
        kl = KubeLibrary(kube_config='test/resources/k3d')
        replicasets = kl.list_namespaced_replica_set_by_pattern('.*', 'test-auto')
        replicasets2 = kl.get_replicasets_in_namespace('.*', 'test-auto')
        self.assertEqual(kl.filter_names(replicasets), kl.filter_replicasets_names(replicasets2))
        self.assertEqual(['nginx-proxy'], kl.filter_names(replicasets))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_persistent_volume_claim')
    def test_list_namespaced_persistent_volume_claim(self, mock_lnp):
        mock_lnp.side_effect = mock_list_pvc
        kl = KubeLibrary(kube_config='test/resources/k3d')
        pvcs = kl.list_namespaced_persistent_volume_claim('default')
        pvcs2 = kl.get_pvc_in_namespace('default')
        self.assertEqual(kl.filter_names(pvcs), pvcs2)
        self.assertEqual(['myclaim'], kl.filter_names(pvcs))

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_service')
    def test_list_namespaced_service(self, mock_service):
        mock_service.side_effect = mock_list_namespaced_services
        kl = KubeLibrary(kube_config='test/resources/k3d')
        ret = kl.list_namespaced_service('default')
        self.assertEqual('test-service', ret[0].metadata.name)

    @mock.patch('kubernetes.client.AppsV1Api.list_namespaced_daemon_set')
    def test_list_namespaced_daemon_set(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_daemonsets
        kl = KubeLibrary(kube_config='test/resources/k3d')
        daemonsets = kl.list_namespaced_daemon_set('default')
        daemonsets2 = kl.get_daemonsets_in_namespace('default')
        self.assertEqual(kl.filter_names(daemonsets), daemonsets2)
        self.assertEqual(['fluentd-elasticsearch'], kl.filter_names(daemonsets))

    @mock.patch('kubernetes.client.ExtensionsV1beta1Api.list_namespaced_ingress')
    def test_list_namespaced_ingress(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_ingresses
        kl = KubeLibrary(kube_config='test/resources/k3d')
        ingresses = kl.list_namespaced_ingress('default')
        ingresses2 = kl.get_ingresses_in_namespace('default')
        self.assertEqual(kl.filter_names(ingresses), ingresses2)
        self.assertEqual(['minimal-ingress'], kl.filter_names(ingresses))

    @mock.patch('kubernetes.client.BatchV1beta1Api.list_namespaced_cron_job')
    def test_list_namespaced_cron_job(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_cronjobs
        kl = KubeLibrary(kube_config='test/resources/k3d')
        cronjobs = kl.list_namespaced_cron_job('default')
        cronjobs2 = kl.get_cron_jobs_in_namespace('default')
        self.assertEqual(kl.filter_names(cronjobs), cronjobs2)
        self.assertEqual(['hello'], kl.filter_names(cronjobs))

    @mock.patch('kubernetes.client.CoreV1Api.read_namespaced_endpoints')
    def test_read_namespaced_endpoints(self, mock_lnp):
        mock_lnp.side_effect = mock_read_namespaced_endpoints
        kl = KubeLibrary(kube_config='test/resources/k3d')
        endpoints = kl.read_namespaced_endpoints('.*', 'default')
        endpoints2 = kl.get_endpoints_in_namespace('.*', 'default')
        self.assertEqual(endpoints.items[0].metadata.name, endpoints2.items[0].metadata.name)
        self.assertEqual('my-service', endpoints.items[0].metadata.name)

    @mock.patch('kubernetes.client.CoreV1Api.list_namespaced_config_map')
    def test_get_configmaps_in_namespace(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_config_map
        kl = KubeLibrary(kube_config='test/resources/k3d')
        configmaps = kl.list_namespaced_config_map_by_pattern('.*', 'default')
        configmaps2 = kl.get_configmaps_in_namespace('.*', 'default')
        self.assertEqual(kl.filter_names(configmaps), kl.filter_configmap_names(configmaps2))
        self.assertEqual(['game-demo'], kl.filter_names(configmaps))

    @mock.patch('kubernetes.client.AutoscalingV1Api.list_namespaced_horizontal_pod_autoscaler')
    def test_list_namespaced_horizontal_pod_autoscaler(self, mock_lnp):
        mock_lnp.side_effect = mock_list_namespaced_hpas
        kl = KubeLibrary(kube_config='test/resources/k3d')
        hpas = kl.list_namespaced_horizontal_pod_autoscaler('default')
        hpas2 = kl.get_hpas_in_namespace('default')
        self.assertEqual(kl.filter_names(hpas), hpas2)
        self.assertEqual(['kubelib-test-test-objects-chart'], kl.filter_names(hpas))

    @mock.patch('kubernetes.client.AutoscalingV1Api.read_namespaced_horizontal_pod_autoscaler')
    def test_read_namespaced_horizontal_pod_autoscaler(self, mock_lnp):
        mock_lnp.side_effect = mock_read_hpa_details_in_namespace
        kl = KubeLibrary(kube_config='test/resources/k3d')
        hpa_details = kl.read_namespaced_horizontal_pod_autoscaler('kubelib-test-test-objects-chart', 'default')
        hpa_details2 = kl.get_hpa_details_in_namespace('kubelib-test-test-objects-chart', 'default')
        self.assertEqual(hpa_details.items.spec.scaleTargetRef.name, hpa_details2.items.spec.scaleTargetRef.name)
        self.assertEqual('kubelib-test-test-objects-chart', hpa_details.items.spec.scaleTargetRef.name)

    @mock.patch('kubernetes.client.AppsV1Api.read_namespaced_daemon_set')
    def test_read_namespaced_daemon_set(self, mock_lnp):
        mock_lnp.side_effect = mock_read_daemonset_details_in_namespace
        kl = KubeLibrary(kube_config='test/resources/k3d')
        daemonset_details = kl.read_namespaced_daemon_set('fluentd-elasticsearch', 'default')
        daemonset_details2 = kl.get_daemonset_details_in_namespace('fluentd-elasticsearch', 'default')
        self.assertEqual(daemonset_details.items.metadata.labels.TestLabel, daemonset_details2.items.metadata.labels.TestLabel)
        self.assertEqual('mytestlabel', daemonset_details.items.metadata.labels.TestLabel)

    @mock.patch('kubernetes.client.CoreV1Api.read_namespaced_service')
    def test_read_namespaced_service(self, mock_lnp):
        mock_lnp.side_effect = mock_read_service_details_in_namespace
        kl = KubeLibrary(kube_config='test/resources/k3d')
        service_details = kl.read_namespaced_service('minimal-ingress', 'default')
        service_details2 = kl.get_service_details_in_namespace('minimal-ingress', 'default')
        self.assertEqual(service_details.items.metadata.labels.Test, service_details2.items.metadata.labels.Test)
        self.assertEqual('mytest', service_details.items.metadata.labels.Test)

    @mock.patch('kubernetes.client.ExtensionsV1beta1Api.read_namespaced_ingress')
    def test_read_namespaced_ingress(self, mock_lnp):
        mock_lnp.side_effect = mock_read_ingress_details_in_namespace
        kl = KubeLibrary(kube_config='test/resources/k3d')
        ingress_details = kl.read_namespaced_ingress('max-ingress', 'default')
        ingress_details2 = kl.get_ingress_details_in_namespace('max-ingress', 'default')
        self.assertEqual(ingress_details.items.metadata.labels.TestLabel, ingress_details2.items.metadata.labels.TestLabel)
        self.assertEqual('mytestlabel', ingress_details.items.metadata.labels.TestLabel)

    @mock.patch('kubernetes.client.BatchV1beta1Api.read_namespaced_cron_job')
    def test_read_namespaced_cron_job(self, mock_lnp):
        mock_lnp.side_effect = mock_read_cron_job_details_in_namespace
        kl = KubeLibrary(kube_config='test/resources/k3d')
        cron_job_details = kl.read_namespaced_cron_job('hello', 'default')
        cron_job_details2 = kl.get_cron_job_details_in_namespace('hello', 'default')
        self.assertEqual(cron_job_details.items.metadata.labels.TestLabel, cron_job_details2.items.metadata.labels.TestLabel)
        self.assertEqual('mytestlabel', cron_job_details.items.metadata.labels.TestLabel)



import json
import re
import ssl
import urllib3

from kubernetes import client, config, dynamic
from os import environ
from robot.api import logger
from robot.api.deco import library
from string import digits, ascii_lowercase
from random import choices

from KubeLibrary.exceptions import BearerTokenWithPrefixException
from KubeLibrary.version import version

# supressing SSL warnings when using self-signed certs
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class DynamicClient(dynamic.DynamicClient):
    @property
    def api_client(self):
        return self.client


@library(scope="GLOBAL", version=version, auto_keywords=True)
class KubeLibrary:
    """KubeLibrary is a Robot Framework test library for Kubernetes.

    The approach taken by this library is to provide easy to access kubernetes objects representation that can
    be then accessed to define highlevel keywords for tests.

    = Kubeconfigs =

    By default ~/.kube/config is used. Kubeconfig location
    can also be passed by setting KUBECONFIG environment variable or as Library argument.

    | ***** Settings *****
    | Library           KubeLibrary          /path/to/kubeconfig

    = Context =

    By default current context from kubeconfig is used. Setting multiple contexts in
    different test suites allows working on multiple clusters.

    | ***** Settings *****
    | Library           KubeLibrary          context=k3d-k3d-cluster2

    = Bearer token authentication =

    It is possible to authenticate using bearer token by passing API url, bearer token and optionally CA certificate.

    | ***** Settings *****
    | Library           KubeLibrary          api_url=%{K8S_API_URL}    bearer_token=%{K8S_TOKEN}    ca_cert=%{K8S_CA_CRT}

    = In cluster execution =

    If tests are supposed to be executed from within cluster, KubeLibrary can be configured to use standard
    token authentication. Just set incluster parameter to True.

    = Auth methods precedence =

    If enabled, auth methods takes precedence in following order:
    1. Incluster
    2. Bearer Token
    3. Kubeconfig

    | ***** Settings *****
    | Library           KubeLibrary          None    True

    """
    def __init__(self, kube_config=None, context=None, api_url=None, bearer_token=None,
                 ca_cert=None, incluster=False, cert_validation=True):
        """KubeLibrary can be configured with several optional arguments.
        - ``kube_config``:
          Path pointing to kubeconfig of target Kubernetes cluster.
        - ``context``:
          Active context. If None current_context from kubeconfig is used.
        - ``api_url``:
          K8s API url, used for bearer token authenticaiton.
        - ``bearer_token``:
          Bearer token, used for bearer token authenticaiton. Do not include 'Bearer ' prefix.
        - ``ca_cert``:
          Optional CA certificate file path, used for bearer token authenticaiton.
        - ``incuster``:
          Default False. Indicates if used from within k8s cluster. Overrides kubeconfig.
        - ``cert_validation``:
          Default True. Can be set to False for self-signed certificates.

        Environment variables:
        - INIT_FOR_LIBDOC_ONLY:
          Set to '1' to generate keyword documentation and skip to load a kube config..
        """
        if "1" == environ.get('INIT_FOR_LIBDOC_ONLY', "0"):
            return
        self.reload_config(kube_config=kube_config, context=context, api_url=api_url, bearer_token=bearer_token,
                           ca_cert=ca_cert, incluster=incluster, cert_validation=cert_validation)

    @staticmethod
    def generate_alphanumeric_str(size):
        """Generates a random alphanumeric string with given size.

        Returns a string.

        - ``size``:
          Desired size of the output string
        """
        return "".join(choices(ascii_lowercase + digits, k=size))

    @staticmethod
    def evaluate_callable_from_k8s_client(attr_name, *args, **kwargs):
        """Evaluates a callable from kubernetes client.

        Returns the output of the client callable.

        - ``attr_name``:
          Callable name
        - ``*args``:
          Positional arguments for argument forwarding
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        attr = getattr(client, attr_name, None)
        assert callable(attr), f"kubernetes.client does not contain {attr_name}!"
        return attr(*args, **kwargs)

    def get_dynamic_resource(self, api_version, kind):
        """Returns a dynamic resource based on the provided api version and kind.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        """
        return self.dynamic.resources.get(api_version=api_version, kind=kind)

    def get(self, api_version, kind, **kwargs):
        """Retrieves resource instances based on the provided parameters.

        Can be optionally given a ``namespace``, ``name``, ``label_selector``, ``body`` and ``field_selector``.

        Returns a resource list.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        resource = self.get_dynamic_resource(api_version, kind)
        return resource.get(**kwargs)

    def create(self, api_version, kind, **kwargs):
        """Creates resource instances based on the provided configuration.

        If the resource is namespaced (ie, not cluster-level), then one of ``namespace``, ``label_selector``, or ``field_selector`` is required.
        If the resource is cluster-level, then one of ``name``, ``label_selector``, or ``field_selector`` is required.
        Can be optionally given a kubernetes manifest (``body``) which respects the above considerations.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        resource = self.get_dynamic_resource(api_version, kind)
        resource.create(**kwargs)

    def delete(self, api_version, kind, **kwargs):
        """Deletes resource instances based on the provided configuration.

        Can be optionally given a ``namespace``, ``name``, ``label_selector``, ``body`` and ``field_selector``.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        resource = self.get_dynamic_resource(api_version, kind)
        resource.delete(**kwargs)

    def patch(self, api_version, kind, **kwargs):
        """Patches resource instances based on the provided parameters.

        Can be optionally given a ``namespace``, ``name``, ``label_selector``, ``body`` and ``field_selector``.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        resource = self.get_dynamic_resource(api_version, kind)
        resource.patch(**kwargs)

    def replace(self, api_version, kind, **kwargs):
        """Replaces resource instances based on the provided parameters.

        Can be optionally given a ``namespace``, ``name``, ``label_selector``, ``body`` and ``field_selector``.

        - ``api_version``:
          Api version of the desired kubernetes resource
        - ``kind``:
          Kind of the desired kubernetes resource
        - ``**kwargs``:
          Keyword arguments for argument forwarding
        """
        resource = self.get_dynamic_resource(api_version, kind)
        resource.replace(**kwargs)

    def reload_config(self, kube_config=None, context=None, api_url=None, bearer_token=None, ca_cert=None, incluster=False, cert_validation=True):
        """Reload the KubeLibrary to be configured with different optional arguments.
           This can be used to connect to a different cluster during the same test.
        - ``kube_config``:
          Path pointing to kubeconfig of target Kubernetes cluster.
        - ``context``:
          Active context. If None current_context from kubeconfig is used.
        - ``api_url``:
          K8s API url, used for bearer token authenticaiton.
        - ``bearer_token``:
          Bearer token, used for bearer token authenticaiton. Do not include 'Bearer ' prefix.
        - ``ca_cert``:
          Optional CA certificate file path, used for bearer token authenticaiton.
        - ``incuster``:
          Default False. Indicates if used from within k8s cluster. Overrides kubeconfig.
        - ``cert_validation``:
          Default True. Can be set to False for self-signed certificates.
        """
        self.api_client = None
        self.cert_validation = cert_validation
        if incluster:
            try:
                config.load_incluster_config()
            except config.config_exception.ConfigException as e:
                logger.error('Are you sure tests are executed from within k8s cluster?')
                raise e
        elif api_url and bearer_token:
            if bearer_token.startswith('Bearer '):
                raise BearerTokenWithPrefixException
            configuration = client.Configuration()
            configuration.api_key["authorization"] = bearer_token
            configuration.api_key_prefix['authorization'] = 'Bearer'
            configuration.host = api_url
            configuration.ssl_ca_cert = ca_cert
            self.api_client = client.ApiClient(configuration)
        else:
            try:
                config.load_kube_config(kube_config, context)
            except TypeError:
                logger.error('Neither KUBECONFIG nor ~/.kube/config available.')

        if not self.api_client:
            self.api_client = client.ApiClient(configuration=client.Configuration().get_default_copy())
        self._add_api('v1', client.CoreV1Api)
        self._add_api('extensionsv1beta1', client.ExtensionsV1beta1Api)
        self._add_api('batchv1', client.BatchV1Api)
        self._add_api('appsv1', client.AppsV1Api)
        self._add_api('batchv1_beta1', client.BatchV1beta1Api)
        self._add_api('custom_object', client.CustomObjectsApi)
        self._add_api('rbac_authv1_api', client.RbacAuthorizationV1Api)
        self._add_api('autoscalingv1', client.AutoscalingV1Api)
        self._add_api('dynamic', DynamicClient)

    def _add_api(self, reference, class_name):
        self.__dict__[reference] = class_name(self.api_client)
        if not self.cert_validation:
            self.__dict__[reference].api_client.rest_client.pool_manager.connection_pool_kw['cert_reqs'] = ssl.CERT_NONE

    def k8s_api_ping(self):
        """Performs GET on /api/v1/ for simple check of API availability.

        Returns tuple of (response data, response status, response headers). Can be used as prerequisite in tests.
        """
        path_params = {}
        query_params = []
        header_params = {}
        auth_settings = ['BearerToken']
        resp = self.v1.api_client.call_api('/api/v1/', 'GET',
                                           path_params,
                                           query_params,
                                           header_params,
                                           response_type='str',
                                           auth_settings=auth_settings,
                                           async_req=False,
                                           _return_http_data_only=False)
        return resp

    def list_namespace(self, label_selector=""):
        """Lists available namespaces.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of namespaces.
        """
        ret = self.v1.list_namespace(watch=False, label_selector=label_selector)
        return ret.items

    def get_namespaces(self, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespace.
        Gets a list of available namespaces.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of namespaces names.
        """
        return self.filter_names(self.list_namespace(label_selector=label_selector))

    def get_healthy_nodes_count(self, label_selector=""):
        """Counts node with KubeletReady and status True.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Can be used to check number of healthy nodes. Can be used as prerequisite in tests.
        """
        ret = self.v1.list_node(watch=False, label_selector=label_selector)
        healthy_nods = []
        for item in ret.items:
            for condition in item.status.conditions:
                if condition.reason == 'KubeletReady' and condition.status == 'True':
                    healthy_nods.append(item.metadata.name)
        return len(healthy_nods)

    def get_pod_names_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_pod_by_pattern.
        Gets pod name matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``name_pattern``:
          Pod name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_pod(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern + '.*')
        return [item.metadata.name for item in ret.items if r.match(item.metadata.name)]

    def list_namespaced_pod_by_pattern(self, name_pattern, namespace, label_selector=""):
        """List pods matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of pods.

        - ``name_pattern``:
          Pod name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_pod(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        pods = [item for item in ret.items if r.match(item.metadata.name)]
        return pods

    def get_pods_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_pod_by_pattern.
        Gets pods matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of pods.

        - ``name_pattern``:
          Pod name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_pod(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        pods = [item for item in ret.items if r.match(item.metadata.name)]
        return pods

    def read_namespaced_pod_log(self, name, namespace, container):
        """Gets container logs of given pod in given namespace.

        Returns logs.

        - ``name``:
          Pod name to check
        - ``namespace``:
          Namespace to check
        - ``container``:
          Container to check
        """
        pod_logs = self.v1.read_namespaced_pod_log(name=name, namespace=namespace, container=container, follow=False)
        return pod_logs

    def get_pod_logs(self, name, namespace, container):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_pod_log.
        Gets container logs of given pod in given namespace.

        Returns logs.

        - ``name``:
          Pod name to check
        - ``namespace``:
          Namespace to check
        - ``container``:
          Container to check
        """
        pod_logs = self.v1.read_namespaced_pod_log(name=name, namespace=namespace, container=container, follow=False)
        return pod_logs

    def list_namespaced_config_map_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Lists configmaps matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of configmaps.

        - ``name_pattern``:
          configmap name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_config_map(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        configmaps = [item for item in ret.items if r.match(item.metadata.name)]
        return configmaps

    def get_configmaps_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_config_map_by_pattern.
        Gets configmaps matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of configmaps.

        - ``name_pattern``:
          configmap name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_config_map(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        configmaps = [item for item in ret.items if r.match(item.metadata.name)]
        return configmaps

    def list_namespaced_service_account_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Lists service accounts matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of service accounts.

        - ``name_pattern``:
          Service Account name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_service_account(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        service_accounts = [item for item in ret.items if r.match(item.metadata.name)]
        return service_accounts

    def get_service_accounts_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_service_account_by_pattern.
        Gets service accounts matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of service accounts.

        - ``name_pattern``:
          Service Account name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_service_account(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        service_accounts = [item for item in ret.items if r.match(item.metadata.name)]
        return service_accounts

    def list_namespaced_deployment_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Gets deployments matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of deployments.

        - ``name_pattern``:
          deployment name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_deployment(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        deployments = [item for item in ret.items if r.match(item.metadata.name)]
        return deployments

    def get_deployments_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_deployment_by_pattern.
        Gets deployments matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of deployments.

        - ``name_pattern``:
          deployment name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_deployment(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        deployments = [item for item in ret.items if r.match(item.metadata.name)]
        return deployments

    def list_namespaced_replica_set_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Lists replicasets matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of  replicasets.

        - ``name_pattern``:
          replicaset name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_replica_set(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        replicasets = [item for item in ret.items if r.match(item.metadata.name)]
        return replicasets

    def get_replicasets_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_replica_set_by_pattern.
        Gets replicasets matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of  replicasets.

        - ``name_pattern``:
          replicaset name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_replica_set(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        replicasets = [item for item in ret.items if r.match(item.metadata.name)]
        return replicasets

    def list_namespaced_job_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Gets jobs matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of jobs.

        - ``name_pattern``:
          job name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1.list_namespaced_job(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        jobs = [item for item in ret.items if r.match(item.metadata.name)]
        return jobs

    def get_jobs_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_job_by_pattern.
        Gets jobs matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of jobs.

        - ``name_pattern``:
          job name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1.list_namespaced_job(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        jobs = [item for item in ret.items if r.match(item.metadata.name)]
        return jobs

    def list_namespaced_secret_by_pattern(self, name_pattern, namespace, label_selector=""):
        """Lists secrets matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of secrets.

        - ``name_pattern``:
          secret name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_secret(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        secrets = [item for item in ret.items if r.match(item.metadata.name)]
        return secrets

    def get_secrets_in_namespace(self, name_pattern, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_secret_by_pattern.
        Gets secrets matching pattern in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of secrets.

        - ``name_pattern``:
          secret name pattern to check
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_secret(namespace, watch=False, label_selector=label_selector)
        r = re.compile(name_pattern)
        secrets = [item for item in ret.items if r.match(item.metadata.name)]
        return secrets

    def filter_names(self, objects):
        """Filter .metadata.name for list of k8s objects.

        Returns list of strings.

        - ``objects``:
          List of k8s objects
        """
        return [obj.metadata.name for obj in objects]

    def filter_by_key(self, objects, key, match):
        """Filter object with key matching value for list of k8s objects.

        Returns list of objects.

        - ``objects``:
          List of k8s objects
        - ``key``:
          Key to match
        - ``match``:
          Value of the key based on which objects will be included
        """
        return [obj for obj in objects if getattr(obj, key) == match]

    def filter_deployments_names(self, deployments):
        """*DEPRECATED* Will be removed in v1.0.0. See examples in TBD.
        Returns list of strings.
        - ``deployments``:
          List of deployments objects
        """
        return self.filter_names(deployments)

    def filter_replicasets_names(self, replicasets):
        """*DEPRECATED* Will be removed in v1.0.0. See examples in TBD.
        Returns list of strings.
        - ``replicasets``:
          List of replicasets objects
        """
        return self.filter_names(replicasets)

    def filter_pods_names(self, pods):
        """*DEPRECATED* Will be removed in v1.0.0. See examples in TBD.
        Filter pod names for list of pods.

        Returns list of strings.

        - ``pods``:
          List of pods objects
        """
        return self.filter_names(pods)

    def filter_service_accounts_names(self, service_accounts):
        """*DEPRECATED* Will be removed in v1.0.0. See examples in TBD.
        Filter service accounts names for list of service accounts.

        Returns list of strings.

        - ``service_accounts``:
          List of service accounts objects
        """
        return self.filter_names(service_accounts)

    def filter_configmap_names(self, configmaps):
        """*DEPRECATED* Will be removed in v1.0.0. See examples in TBD.
        Filter configmap  names for list of configmaps.
        Returns list of strings.
        - ``configmaps``:
          List of configmap objects
        """
        return self.filter_names(configmaps)

    def filter_endpoints_names(self, endpoints):
        """Filter endpoints names for list of endpoints.
        Returns list of strings.
        - ``endpoints``:
        List of endpoint objects
        """
        return self.filter_names(endpoints.items)

    @staticmethod
    def filter_pods_containers_by_name(pods, name_pattern):
        """Filters pods containers by name for given list of pods.

        Returns lists of containers (flattens).

        - ``pods``:
          List of pods objects
        """
        containers = []
        r = re.compile(name_pattern)
        for pod in pods:
            for container in pod.spec.containers:
                if r.match(container.name):
                    containers.append(container)
        return containers

    @staticmethod
    def filter_containers_images(containers):
        """Filters container images for given lists of containers.

        Returns list of images.

        - ``containers``:
          List of containers
        """
        return [container.image for container in containers]

    @staticmethod
    def filter_containers_resources(containers):
        """Filters container resources for given lists of containers.

        Returns list of resources.

        - ``containers``:
          List of containers
        """
        return [container.resources for container in containers]

    @staticmethod
    def filter_pods_containers_statuses_by_name(pods, name_pattern):
        """Filters pods containers statuses by container name for given list of pods.

        Returns lists of containers statuses.

        - ``pods``:
          List of pods objects
        """
        container_statuses = []
        r = re.compile(name_pattern)
        for pod in pods:
            for container_status in pod.status.container_statuses:
                if r.match(container_status.name):
                    container_statuses.append(container_status)
        return container_statuses

    def read_namespaced_pod_status(self, name, namespace):
        """Reads pod status in given namespace.

        - ``name``:
          Name of pod.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_pod_status(name, namespace)
        return ret.status

    def get_pod_status_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_pod_status.

        - ``name``:
          Name of pod.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_pod_status(name, namespace)
        return ret.status.phase

    @staticmethod
    def assert_pod_has_labels(pod, labels_json):
        """Assert pod has labels.

        Returns True/False

        - ``pod``:
          Pod object.
        - ``labels_json``:
          JSON representing labels
        """
        try:
            labels = json.loads(labels_json)
            for k, v in labels.items():
                if pod.metadata.labels and k in pod.metadata.labels:
                    if pod.metadata.labels[k] != v:
                        logger.error(f'Label "{k}" value "{v}" not matching actual "{pod.metadata.labels[k]}"')
                        return False
                else:
                    logger.error(f'Label "{k}" not found in actual')
                    return False
            return True
        except json.JSONDecodeError:
            logger.error(f'Failed parsing Pod Labels JSON:{labels_json}')
            return False

    @staticmethod
    def assert_pod_has_annotations(pod, annotations_json):
        """Assert pod has annotations.

        Returns True/False

        - ``pod``:
          Pod object.
        - ``annotations_json``:
          JSON representing annotations
        """
        try:
            annotations = json.loads(annotations_json)
            for k, v in annotations.items():
                if pod.metadata.annotations and k in pod.metadata.annotations:
                    if pod.metadata.annotations[k] != v:
                        logger.error(f'Annotation "{k}" value "{v}" not matching actual "{pod.metadata.annotations[k]}"')
                        return False
                else:
                    logger.error(f'Annotation "{k}" not found in actual')
                    return False
            return True
        except json.JSONDecodeError:
            logger.error(f'Failed parsing Pod Annotations JSON:{annotations_json}')
            return False

    @staticmethod
    def assert_container_has_env_vars(container, env_vars_json):
        """Assert container has env vars.

        Returns True/False

        - ``container``:
          Container object.
        - ``env_var_json``:
          JSON representing env vars i.e.: {"EXAMPLE_VAR": "examplevalue"}
        """
        try:
            env_vars = json.loads(env_vars_json)
            for k, v in env_vars.items():
                found = False
                for ev in container.env:
                    if k == ev.name and v == ev.value:
                        found = True
                        break
                    elif k == ev.name and v != ev.value:
                        logger.error(f'Env var "{k}" value "{v}" not matching actual "{ev.value}"')
                        return False
                if not found:
                    logger.error(f'Env var "{k}" not found in actual')
                    return False
            return True
        except json.JSONDecodeError:
            logger.error(f'Failed parsing Container Env Var JSON:{env_vars_json}')
            return False

    def list_namespaced_service(self, namespace, label_selector=""):
        """Gets services in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_service(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_services_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_service.

        Gets services in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_service(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_service(self, name, namespace):
        """Gets service details in given namespace.

        Returns Service object representation. Can be accessed using

        | Should Be Equal As integers    | ${service_details.spec.ports[0].port}    | 8080 |

        - ``name``:
          Name of service.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_service(name, namespace)
        return ret

    def get_service_details_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_service.

        Gets service details in given namespace.

        Returns Service object representation. Can be accessed using

        | Should Be Equal As integers    | ${service_details.spec.ports[0].port}    | 8080 |

        - ``name``:
          Name of service.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_service(name, namespace)
        return ret

    def list_namespaced_horizontal_pod_autoscaler(self, namespace, label_selector=""):
        """Gets Horizontal Pod Autoscalers in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.autoscalingv1.list_namespaced_horizontal_pod_autoscaler(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_hpas_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_horizontal_pod_autoscaler.

        Gets Horizontal Pod Autoscalers in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.autoscalingv1.list_namespaced_horizontal_pod_autoscaler(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_horizontal_pod_autoscaler(self, name, namespace):
        """Gets Horizontal Pod Autoscaler details in given namespace.

        Returns Horizontal Pod Autoscaler object representation. Can be accessed using

        | Should Be Equal As integers    | ${hpa_details.spec.target_cpu_utilization_percentage}    | 50 |

        - ``name``:
          Name of Horizontal Pod Autoscaler
        - ``namespace``:
          Namespace to check
        """
        ret = self.autoscalingv1.read_namespaced_horizontal_pod_autoscaler(name, namespace)
        return ret

    def get_hpa_details_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_horizontal_pod_autoscaler.

        Gets Horizontal Pod Autoscaler details in given namespace.

        Returns Horizontal Pod Autoscaler object representation. Can be accessed using

        | Should Be Equal As integers    | ${hpa_details.spec.target_cpu_utilization_percentage}    | 50 |

        - ``name``:
          Name of Horizontal Pod Autoscaler
        - ``namespace``:
          Namespace to check
        """
        ret = self.autoscalingv1.read_namespaced_horizontal_pod_autoscaler(name, namespace)
        return ret

    def read_namespaced_endpoints(self, name, namespace):
        """Gets endpoint details in given namespace.

        Returns Endpoint object representation. Can be accessed using

        | Should Match    | ${endpoint_details.subsets[0].addresses[0].target_ref.name}    | pod-name-123456 |

        - ``name``:
          Name of endpoint.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_endpoints(name, namespace)
        return ret

    def get_endpoints_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_endpoints.

        Gets endpoint details in given namespace.

        Returns Endpoint object representation. Can be accessed using

        | Should Match    | ${endpoint_details.subsets[0].addresses[0].target_ref.name}    | pod-name-123456 |

        - ``name``:
          Name of endpoint.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_endpoints(name, namespace)
        return ret

    def list_namespaced_persistent_volume_claim(self, namespace, label_selector=""):
        """Gets pvcs in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_persistent_volume_claim(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_pvc_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_persistent_volume_claim.

        Gets pvcs in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.list_namespaced_persistent_volume_claim(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_persistent_volume_claim(self, name, namespace):
        """Gets PVC details in given namespace.

        Returns PVC object representation. Can be accessed using

        | Should Be Equal As strings    | ${pvc.status.capacity.storage}    | 1Gi |

        - ``name``:
          Name of PVC.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_persistent_volume_claim(name, namespace)
        return ret

    def get_pvc_capacity(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_persistent_volume_claim.

        Gets PVC details in given namespace.

        Returns PVC object representation. Can be accessed using

        | Should Be Equal As strings    | ${pvc.status.capacity.storage}    | 1Gi |

        - ``name``:
          Name of PVC.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.read_namespaced_persistent_volume_claim(name, namespace)
        return ret

    def get_kubelet_version(self, label_selector=""):
        """Gets list of kubelet versions on each node.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.
        """
        ret = self.v1.list_node(watch=False, label_selector=label_selector)
        return [item.status.node_info.kubelet_version for item in ret.items]

    def create_namespaced_service_account(self, namespace, body):
        """Creates service account in a namespace

        Returns created service account

        - ``body``:
          Service Account object.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.create_namespaced_service_account(namespace=namespace, body=body)
        return ret

    def create_service_account_in_namespace(self, namespace, body):
        """*DEPRECATED* Will be removed in v1.0.0. Use create_namespaced_service_account.

        Creates service account in a namespace

        Returns created service account

        - ``body``:
          Service Account object.
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.create_namespaced_service_account(namespace=namespace, body=body)
        return ret

    def delete_namespaced_service_account(self, name, namespace):
        """Deletes service account in a namespace

        Returns V1status


        - ``name``:
          Service Account name
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.delete_namespaced_service_account(name=name, namespace=namespace)
        return ret

    def delete_service_account_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use delete_namespaced_service_account.

        Deletes service account in a namespace

        Returns V1status


        - ``name``:
          Service Account name
        - ``namespace``:
          Namespace to check
        """
        ret = self.v1.delete_namespaced_service_account(name=name, namespace=namespace)
        return ret

    def get_healthcheck(self, endpoint='/readyz', verbose=False):
        """Performs GET on /readyz or /livez for simple health check.

        Can be used to verify the readiness/current status of the API server
        Returns tuple of (response data, response status and response headers)

        - ``endpoint``:
            /readyz, /livez or induvidual endpoints like '/livez/etcd'. defaults to /readyz
        - ``verbose``:
            More detailed output.

        https://kubernetes.io/docs/reference/using-api/health-checks

        """
        path_params = {}
        query_params = []
        header_params = {}
        auth_settings = ['BearerToken']
        if not (endpoint.startswith('/readyz') or endpoint.startswith('/livez')):
            raise RuntimeError(f'{endpoint} does not start with "/readyz" or "/livez"')
        endpoint = endpoint if not verbose else endpoint + '?verbose'
        resp = self.v1.api_client.call_api(endpoint, 'GET',
                                           path_params,
                                           query_params,
                                           header_params,
                                           response_type='str',
                                           auth_settings=auth_settings,
                                           async_req=False,
                                           _return_http_data_only=False)
        return resp

    def list_namespaced_ingress(self, namespace, label_selector=""):
        """Gets ingresses in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value
        Returns list of strings.
        - ``namespace``:
          Namespace to check
        """
        ret = self.extensionsv1beta1.list_namespaced_ingress(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_ingresses_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_ingress.

        Gets ingresses in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value
        Returns list of strings.
        - ``namespace``:
          Namespace to check
        """
        ret = self.extensionsv1beta1.list_namespaced_ingress(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_ingress(self, name, namespace):
        """Gets ingress details in given namespace.

        Returns Ingress object representation.
          Name of ingress.
        - ``namespace``:
          Namespace to check
        """
        ret = self.extensionsv1beta1.read_namespaced_ingress(name, namespace)
        return ret

    def get_ingress_details_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_ingress.

        Gets ingress details in given namespace.

        Returns Ingress object representation.
          Name of ingress.
        - ``namespace``:
          Namespace to check
        """
        ret = self.extensionsv1beta1.read_namespaced_ingress(name, namespace)
        return ret

    def list_namespaced_cron_job(self, namespace, label_selector=""):
        """Gets cron jobs in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.list_namespaced_cron_job(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_cron_jobs_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_cron_job.

        Gets cron jobs in given namespace.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of strings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.list_namespaced_cron_job(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_cron_job(self, name, namespace):
        """Gets cron job details in given namespace.

        Returns Cron job object representation.

        - ``name``:
          Name of cron job.
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.read_namespaced_cron_job(name, namespace)
        return ret

    def get_cron_job_details_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_cron_job.

        Gets cron job details in given namespace.

        Returns Cron job object representation.

        - ``name``:
          Name of cron job.
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.read_namespaced_cron_job(name, namespace)
        return ret

    def list_namespaced_daemon_set(self, namespace, label_selector=""):
        """Gets a list of available daemonsets.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of deaemonsets.

        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_daemon_set(namespace, watch=False, label_selector=label_selector)
        return [item for item in ret.items]

    def get_daemonsets_in_namespace(self, namespace, label_selector=""):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_daemon_set.

        Gets a list of available daemonsets.

        Can be optionally filtered by label. e.g. label_selector=label_key=label_value

        Returns list of deaemonsets.

        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.list_namespaced_daemon_set(namespace, watch=False, label_selector=label_selector)
        return [item.metadata.name for item in ret.items]

    def read_namespaced_daemon_set(self, name, namespace):
        """Gets deamonset details in given namespace.

        Returns daemonset object representation.

        - ``name``:
          Name of the daemonset
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.read_namespaced_daemon_set(name, namespace)
        return ret

    def get_daemonset_details_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use read_namespaced_daemon_set.

        Gets deamonset details in given namespace.

        Returns daemonset object representation.

        - ``name``:
          Name of the daemonset
        - ``namespace``:
          Namespace to check
        """
        ret = self.appsv1.read_namespaced_daemon_set(name, namespace)
        return ret

    def list_cluster_role(self):
        """Gets a list of cluster_roles.

        Returns list of cluster_roles.
        """
        ret = self.rbac_authv1_api.list_cluster_role(watch=False)
        return [item for item in ret.items]

    def get_cluster_roles(self):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_cluster_role.

        Gets a list of cluster_roles.

        Returns list of cluster_roles.
        """
        ret = self.rbac_authv1_api.list_cluster_role(watch=False)
        return [item.metadata.name for item in ret.items]

    def list_cluster_role_binding(self):
        """Gets a list of cluster_role_bindings.

        Returns list of cluster_role_bindings.
        """
        ret = self.rbac_authv1_api.list_cluster_role_binding(watch=False)
        return [item for item in ret.items]

    def get_cluster_role_bindings(self):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_cluster_role_binding.

        Gets a list of cluster_role_bindings.

        Returns list of cluster_role_bindings.
        """
        ret = self.rbac_authv1_api.list_cluster_role_binding(watch=False)
        return [item.metadata.name for item in ret.items]

    def list_namespaced_role(self, namespace):
        """Gets roles in given namespace.

        Returns list of roles.

        - ``namespace``:
          Namespace to check
        """
        ret = self.rbac_authv1_api.list_namespaced_role(namespace, watch=False)
        return [item for item in ret.items]

    def get_roles_in_namespace(self, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_role.

        Gets roles in given namespace.

        Returns list of roles.

        - ``namespace``:
          Namespace to check
        """
        ret = self.rbac_authv1_api.list_namespaced_role(namespace, watch=False)
        return [item.metadata.name for item in ret.items]

    def list_namespaced_role_binding(self, namespace):
        """Gets role_bindings in given namespace.

        Returns list of role_bindings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.rbac_authv1_api.list_namespaced_role_binding(namespace, watch=False)
        return [item for item in ret.items]

    def get_role_bindings_in_namespace(self, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_namespaced_role_binding.

        Gets role_bindings in given namespace.

        Returns list of role_bindings.

        - ``namespace``:
          Namespace to check
        """
        ret = self.rbac_authv1_api.list_namespaced_role_binding(namespace, watch=False)
        return [item.metadata.name for item in ret.items]

    def list_cluster_custom_object(self, group, version, plural):
        """Lists cluster level custom objects.

        Returns an object.

        - ``group``:
          API Group, e.g. 'k8s.cni.cncf.io'
        - ``version``:
          API version, e.g. 'v1'
        - ``plural``:
          e.g. 'network-attachment-definitions'

        As in ``GET /apis/{group}/{version}/{plural}``

        https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        return self.custom_object.list_cluster_custom_object(group, version, plural)

    def list_cluster_custom_objects(self, group, version, plural):
        """*DEPRECATED* Will be removed in v1.0.0. Use list_cluster_custom_object.

        Lists cluster level custom objects.

        Returns an object.

        - ``group``:
          API Group, e.g. 'k8s.cni.cncf.io'
        - ``version``:
          API version, e.g. 'v1'
        - ``plural``:
          e.g. 'network-attachment-definitions'

        As in ``GET /apis/{group}/{version}/{plural}``

        https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        return self.custom_object.list_cluster_custom_object(group, version, plural)

    def get_cluster_custom_object(self, group, version, plural, name):
        """Get cluster level custom object.

        Returns an object.

        - ``group``:
          API Group, e.g. 'scheduling.k8s.io'
        - ``version``:
          API version, e.g. 'v1'
        - ``plural``:
          e.g. 'priorityclasses'
        - ``name``:
          e.g. 'system-node-critical'

        As in ``GET /apis/{group}/{version}/{plural}/{name}``

        https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        return self.custom_object.get_cluster_custom_object(group, version, plural, name)

    def get_namespaced_custom_object(self, group, version, namespace, plural, name):
        """Get custom object in namespace.

        Returns an object.

        - ``group``:
          API Group, e.g. 'k8s.cni.cncf.io'
        - ``version``:
          API version, e.g. 'v1'
        - ``namespace``:
          Namespace, e.g. 'default'
        - ``plural``:
          e.g. 'network-attachment-definitions'
        - ``name``:
          e.g. 'my-network'

        As in ``GET /apis/{group}/{version}/namespaces/{namespace}/{plural}/{name}``

        https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        return self.custom_object.get_namespaced_custom_object(group, version, namespace, plural, name)

    def get_custom_object_in_namespace(self, group, version, namespace, plural, name):
        """*DEPRECATED* Will be removed in v1.0.0. Use get_namespaced_custom_object.

        Get custom object in namespace.

        Returns an object.

        - ``group``:
          API Group, e.g. 'k8s.cni.cncf.io'
        - ``version``:
          API version, e.g. 'v1'
        - ``namespace``:
          Namespace, e.g. 'default'
        - ``plural``:
          e.g. 'network-attachment-definitions'
        - ``name``:
          e.g. 'my-network'

        As in ``GET /apis/{group}/{version}/namespaces/{namespace}/{plural}/{name}``

        https://github.com/kubernetes-client/python/blob/master/kubernetes/README.md
        """
        return self.custom_object.get_namespaced_custom_object(group, version, namespace, plural, name)

    def create_namespaced_cron_job(self, namespace, body):
        """Creates cron_job in a namespace

        Returns created cron_job
        - ``body``:
          Cron_job object.
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.create_namespaced_cron_job(namespace=namespace, body=body)
        return ret

    def create_cron_job_in_namespace(self, namespace, body):
        """*DEPRECATED* Will be removed in v1.0.0. Use create_namespaced_cron_job.

        Creates cron_job in a namespace

        Returns created cron_job
        - ``body``:
          Cron_job object.
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.create_namespaced_cron_job(namespace=namespace, body=body)
        return ret

    def delete_namespaced_cron_job(self, name, namespace):
        """Deletes cron_job in a namespace

        Returns V1 status
        - ``name``:
          Cron Job name
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.delete_namespaced_cron_job(name=name, namespace=namespace)
        return ret

    def delete_cron_job_in_namespace(self, name, namespace):
        """*DEPRECATED* Will be removed in v1.0.0. Use delete_namespaced_cron_job.

        Deletes cron_job in a namespace

        Returns V1 status
        - ``name``:
          Cron Job name
        - ``namespace``:
          Namespace to check
        """
        ret = self.batchv1_beta1.delete_namespaced_cron_job(name=name, namespace=namespace)
        return ret



from pkg_resources import parse_requirements
from pathlib import Path
from setuptools import setup

exec(open("src/KubeLibrary/version.py").read())

with open("README.md", "r") as fh:
    long_description = fh.read()

with Path("requirements.txt").open() as requirements:
    install_requires = [
        str(requirement)
        for requirement in parse_requirements(requirements)
    ]

setup(
    name="robotframework-kubelibrary",
    version=version,
    author="Michał Wcisło",
    author_email="mwcislo999@gmail.com",
    description="Kubernetes library for Robot Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/devopsspiral/KubeLibrary",
    license="MIT",
    packages=["KubeLibrary"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Testing",
    ],
    keywords="robotframework testing test automation kubernetes",
    python_requires='>=3.6',
    package_dir={'': 'src'},
    install_requires=install_requires,
)
